from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from rest_framework.generics import GenericAPIView
from django.db import transaction
from .jwt_token import JwtToken
from .utils import MyJsonResponse, calculate_md5
from .models import User
from .serializers import AdminLoginSerializer
import logging


class AdminLoginView(GenericAPIView):
    """后台登录的视图类"""

    serializer_class = AdminLoginSerializer

    @transaction.atomic
    def post(self, request):
        """后台登录接口"""
        res = MyJsonResponse()
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        username = serializer.data.get("username")
        password = serializer.data.get("password")
        user = User.objects.filter(username=username).first()

        if not user:
            res.update(msg="User not found.", code=2)
            return Response(res.data, status=status.HTTP_404_NOT_FOUND)
        if not user.check_password(password):
            res.update(msg="Wrong password.", code=2)
            return Response(res.data, status=status.HTTP_400_BAD_REQUEST)
        payload = {"id": user.id, "username": username}
        logging.debug(payload)
        jwt_token = JwtToken().encode_user(payload)
        user.save()
        res.update(data={"token": jwt_token})
        return Response({"token": jwt_token}, status=status.HTTP_200_OK)



from django.db import models
from .utils import calculate_md5


class User(models.Model):
    username = models.CharField(max_length=191, blank=True, null=True, unique=True)
    password_hash = models.CharField(max_length=191, blank=True, null=True)
    created_at = models.DateTimeField(blank=True, null=True)
    updated_at = models.DateTimeField(blank=True, null=True)
    phone_number = models.CharField(max_length=191, blank=True, null=True)
    active = models.BooleanField(default=None, null=True, blank=True)
    email = models.EmailField(max_length=255, blank=True, null=True)
    exten = models.CharField(max_length=191, blank=True, null=True)
    exten_pwd = models.CharField(max_length=191, blank=True, null=True)
    voip = models.BooleanField(default=False)
    department = models.CharField(max_length=255, blank=True, null=True)
    can_create_free_order = models.BooleanField(default=False)
    dingtalk_id = models.CharField(max_length=191, blank=True, null=True)
    taobao_access_code = models.CharField(max_length=191, blank=True, null=True)
    name = models.CharField(max_length=191, blank=True, null=True)
    workshop = models.CharField(max_length=191, blank=True, null=True)
    is_web_and_wx_customer_service = models.BooleanField(
        default=None, null=True, blank=True
    )
    dingtalk_open_id = models.CharField(max_length=50, blank=True, null=True)
    dingtalk_union_id = models.CharField(max_length=50, blank=True, null=True)
    homepage_url = models.URLField(max_length=255, blank=True, null=True)
    qiye_weixin_open_userid = models.CharField(
        max_length=255, blank=True, null=True, verbose_name="企业微信open_userid"
    )
    qiye_weixin_userid = models.CharField(
        max_length=255, blank=True, null=True, verbose_name="企业微信userid"
    )
    work_status = models.CharField(max_length=255, default="下班")

    USERNAME_FIELD = "username"
    REQUIRED_FIELDS = []

    class Meta:
        db_table = "users"
        ordering = ["id"]

    def set_password(self, raw_password):
        self.password_hash = calculate_md5(self.username, raw_password)

    def check_password(self, raw_password):
        return self.password_hash == calculate_md5(self.username, raw_password)

    @property
    def is_anonymous(self):
        return False

    @property
    def is_authenticated(self):
        return True



import jwt
import logging
from datetime import datetime
from django.conf import settings
from django.db.models import Model
from jwt.exceptions import ExpiredSignatureError, InvalidSignatureError, DecodeError


class JwtToken:

    def __init__(self) -> None:
        self.key = settings.JWT_SETTINGS["SIGNING_KEY"]
        self.algrithms = settings.JWT_SETTINGS["ALGORITHMS"]
        self.algrithm = settings.JWT_SETTINGS["ALGORITHMS"][0]
        self.header_type = settings.JWT_SETTINGS["AUTH_HEADER_TYPES"]
        self.header_name = settings.JWT_SETTINGS["AUTH_HEADER_NAME"]
        self.options = {
            "verify_signature": settings.JWT_SETTINGS["VERIFY_SIGNATURE"],
            "verify_exp": settings.JWT_SETTINGS["VERIFY_EXP"],
            "require": settings.JWT_SETTINGS["REQUIRE"],
        }

    def encode(self, payload: dict) -> str:
        tmp = {
            "exp": datetime.utcnow() + settings.JWT_SETTINGS["ACCESS_TOKEN_LIFETIME"]
        }
        payload.update(tmp)
        return jwt.encode(payload=payload, key=self.key, algorithm=self.algrithm)

    def decode(self, s: str) -> tuple:
        try:
            res = jwt.decode(
                jwt=s, key=self.key, algorithms=self.algrithms, options=self.options
            )
            return res, ""
        except ExpiredSignatureError as e:
            return None, "Token expired."
        except InvalidSignatureError as e:
            return None, "Token is not valid."
        except DecodeError as e:
            return None, "Not enough segments."

    def encode_user(self, payload: dict) -> str:
        if not isinstance(payload, dict):
            raise ValueError("Payload must be a dict type.")
        if "id" not in payload or "username" not in payload:
            raise KeyError("Payload must contain the 'id' and 'username' fields.")
        return self.encode(payload)

    def decode_user(self, s: str, User: Model) -> tuple:
        try:
            obj, msg = self.decode(s)
            if not obj:
                return obj, msg
            user = (
                User.objects.values("id", "username", "password_hash")
                .filter(id=obj["id"])
                .first()
            )
            if not user:
                return None, "The account does not exist."
            if user.get("username") != obj["username"]:
                return None, "The token has been refreshed."
            return User.objects.filter(id=obj["id"]).first(), ""
        except Exception as e:
            logging.error(f"Error decoding jwt: {e}")
            logging.exception(e)
            return None, str(e)

    def check_headers_jwt(self, target: str) -> tuple:
        target_ls = target.strip().split(" ")
        if len(target_ls) != 2:
            return None, "Invalid Authorization header."
        header_type, value = target_ls
        if header_type != self.header_type:
            return None, "Invalid header type."
        return value, ""



import hashlib


def calculate_md5(username, password):
    data = ":".join([username, "Piehouse", password])
    hash_object = hashlib.md5(data.encode())
    md5_hash = hash_object.hexdigest()
    return md5_hash


class MyJsonResponse:
    def __init__(self):
        self.data = {"msg": "", "code": 0, "data": {}}

    def update(self, msg=None, code=None, data=None):
        if msg:
            self.data["msg"] = msg
        if code:
            self.data["code"] = code
        if data:
            self.data["data"] = data



from django.urls import path, include
from rest_framework.routers import DefaultRouter
from .views import AdminLoginView

router = DefaultRouter()

urlpatterns = [
    path("", include(router.urls)),
    path("login", AdminLoginView.as_view(), name="admin-login"),
]



from django.db import models


class BaseModel(models.Model):
    id = models.AutoField(primary_key=True, help_text="主键，自增的整数")  # 主键ID
    created_at = models.DateTimeField(null=True, blank=True, verbose_name="创建时间")
    updated_at = models.DateTimeField(null=True, blank=True, verbose_name="更新时间")

    class Meta:
        abstract = True



from rest_framework import serializers
from .models import User
from rest_framework.serializers import ModelSerializer
from extensions.BaseSerializer import BaseModelSerializer


class UserViewsetSerializer(BaseModelSerializer, ModelSerializer):
    class Meta:
        model = User
        fields = [
            "id",
            "username",
            "password_hash",
            "created_at",
            "updated_at",
            "phone_number",
            "active",
            "email",
            "exten",
            "exten_pwd",
            "voip",
            "department",
            "can_create_free_order",
            "dingtalk_id",
            "taobao_access_code",
            "name",
            "workshop",
            "is_web_and_wx_customer_service",
            "dingtalk_open_id",
            "dingtalk_union_id",
            "homepage_url",
            "qiye_weixin_open_userid",
            "qiye_weixin_userid",
            "work_status",
        ]
        read_only_fields = ["id", "created_at", "updated_at"]


class AdminLoginSerializer(serializers.Serializer):
    username = serializers.CharField(required=True, max_length=64, label="账号")
    password = serializers.CharField(required=True, max_length=64, label="密码")



from setuptools import setup, find_packages

setup(
    name="pyf_login",
    version="0.21",
    packages=find_packages(),
    install_requires=[
        "djangorestframework",
        "PyJWT",
    ],
    include_package_data=True,
    description="A reusable Django login package",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="",
    author="ycx",
    author_email="ycx3030@126.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)
